#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
update_dics_rc.py

script de mise à jour des dictionnaires zephir en version proposed

- faire un clone de https://dev-eole.ac-dijon.fr/zephir-parc/
- modifier le chemin ZEPHIR_DIR pour pointer sur le répertoire du projet
- lancer le script pour récupérer les dictionnaires
- vérifier et commiter les modifications dans data/dictionnaires du projet zephir-parc

- recompiler le paquet zephir-parc

"""

import os, sys, shutil
from glob import glob
from subprocess import Popen, PIPE
from eolepack.eolerc import list_packages

# A CONFIGURER : CHEMIN VERS LE REPERTOIRE DES DICTIONNAIRES (checkout de zephir-parc)
ZEPHIR_DIR = '/home/bruno/git/zephir/parc_22'

EOLE_VERS = "2.2"
SERVEUR_GIT = 'http://dev-eole.ac-dijon.fr'
DEST_DIR = os.path.abspath('/tmp/dicos_git')
# ditribution actuelle : master par défaut
BRANCHE_DIST = "2.2"
REPOS_DIST = '%s/git' % (SERVEUR_GIT)
# REPOS_OUTILS = '%s/Outils' % (SERVEUR_GIT) -> arbo git à plat
ZEPHIR_DICT_DIR = os.path.join(ZEPHIR_DIR, 'data', 'dictionnaires')

TYPE_PAQ = 'security'

if '-p' in sys.argv:
    print "\nRécupération des dictionnaires dans les derniers paquets proposed (%s)" % EOLE_VERS
    TYPE_PAQ = 'proposed'
else:
    print "\nRécupération des dictionnaires dans les derniers paquets stables (%s)" % EOLE_VERS

def run_cmd(args, show_output = False):
    running_cmd = Popen(args, stdout=PIPE, stderr=PIPE)
    if running_cmd.wait() != 0:
        data = running_cmd.communicate()
        print data[0]
        print u"! Erreur lors de l'exécution de la commande suivante: %s\n" % " ".join(args)
        print data[1]
        sys.exit(1)
    if show_output:
        print running_cmd.communicate()[0]

# dictionnaire 'nom de paquet' : (, chemin svn, {destination de copie:[fichiers_xml]}, nom_projet_git)
CONF_PACKS = {'conf-amon':('', {}, ''),
              'conf-eclair':('', {}, ''),
              'conf-eolebase':('', {}, ''),
              'conf-eolevz':('eole/dicos', {'amonecole-%s' % EOLE_VERS: []}, ''),
              'conf-horus':('', {}, ''),
              'conf-preludemanager':('', {}, ''),
              'conf-scribe':('', {}, ''),
              'conf-sentinelle':('', {}, ''),
              'conf-seshat':('', {}, ''),
              'conf-sphynx':('', {}, ''),
              'conf-zephir':('', {}, ''),
              'conf-zephirlogs':('', {}, 'conf-zephirlog'),
              'eole-bacula':('', {'scribe-%s' % EOLE_VERS: [],
                                  'horus-%s' % EOLE_VERS: [],
                                  'eclair-%s' % EOLE_VERS: []}, ''),
              'eole-mysql':('', {'scribe-%s' % EOLE_VERS: ['scribe_mysql.xml'],
                                 'horus-%s' % EOLE_VERS: ['2_mysql.xml']}, ''),
              'eole-common':('', {}, ''),
              'eole-posh':('eole/dicos', {'scribe-%s' % EOLE_VERS: []}, ''),
              'client-common':('', {'eclair-%s' % EOLE_VERS: []}, ''),
              'ajax-portail':('eole', {'scribe-%s' % EOLE_VERS: []}, ''),
              'envole-cdt':('', {'scribe-%s' % EOLE_VERS: []}, 'cdt'),
              'envole-grr':('', {'scribe-%s' % EOLE_VERS: []}, 'grr'),
              'envole-calendar':('', {'scribe-%s' % EOLE_VERS: []}, 'webcalendar'),
              'eole-taskfreak':('', {'scribe-%s' % EOLE_VERS: []}, 'taskfreak'),
              'eole-sso':('', {}, ''),
             }

# récupération de la liste des paquets RC
print u"\nMise à jour de la liste des paquets ..."

liste_paqs = []
for pack in list_packages("http://test-eoleng.ac-dijon.fr/eoleng/eole-%s-%s/source/" % (EOLE_VERS, TYPE_PAQ)):
    if pack[0] in CONF_PACKS:
        liste_paqs.append(pack)

print u"Export des dictionnaires ...\n"

if os.path.isdir(DEST_DIR):
    shutil.rmtree(DEST_DIR)
    os.makedirs(DEST_DIR)
for name, tag in liste_paqs:
    tag = tag.rsplit("-")[1]
    print u" - %s (tag %s)" % (name, tag)
    if CONF_PACKS[name][0] == '':
        path_dicos = 'dicos'
    else:
        path_dicos = CONF_PACKS[name][0]
    #    # chemin par défaut (conf module)
    #    svn_path = "/".join([REPOS_DIST, name.lstrip('conf-'), name, 'tags', tag, 'dicos'])
    #else:
    if CONF_PACKS[name][2] == '':
        git_path = "/".join([REPOS_DIST, name])
    else:
        # nom de projet git != nom de paquet
        git_path = "/".join([REPOS_DIST, CONF_PACKS[name][2]])

    # lancement du clone GIT sur la branche de la distribution
    dest_path = os.path.join(DEST_DIR, name)
    export_path = os.path.join(DEST_DIR, 'git', name)
    run_cmd(['/usr/bin/git', 'clone', '-b', BRANCHE_DIST, git_path, export_path])
    # conservation seulement du répertoire des dictionnaires
    run_cmd(['/bin/mv', '%s/%s' % (export_path, path_dicos), dest_path])

shutil.rmtree(os.path.join(DEST_DIR, 'git'))
# test xmllint de tous les fichiers xml
print u"\nVérification de la syntaxe (xmllint) ...\n"
for fic_xml in glob("%s/*/*.xml" % DEST_DIR):
    run_cmd(['/usr/bin/xmllint', '-noout', fic_xml])

print u"\nMise à jour des dictionnaire Zephir ...\n"
# définition des copies à effectuer

copies = []
for paq in liste_paqs:
    name = paq[0]
    # copies automatiques pour les paquets conf-<module>
    if name.startswith('conf-'):
        mod_name = name.lstrip('conf-')
        dest = os.path.join(ZEPHIR_DICT_DIR, "%s-%s/" % (mod_name, EOLE_VERS))
        if os.path.isdir(dest):
            for fic_xml in glob(os.path.join(DEST_DIR, name,'*.xml')):
                copies.append((fic_xml, dest))
        else:
            print  u"** %s : Fichiers non copiés (module non géré dans zephir)**" % name
    if CONF_PACKS[name][1] != {}:
        # copies spécifiques
        dests = CONF_PACKS[name][1].keys()
        for mod_dest in dests:
            dest = os.path.join(ZEPHIR_DICT_DIR, mod_dest)
            if os.path.isdir(dest):
                if CONF_PACKS[name][1][mod_dest] != []:
                    # dictionnaires spécifiés
                    for dict_name in CONF_PACKS[name][1][mod_dest]:
                        fic_xml = os.path.join(DEST_DIR, name, dict_name)
                        copies.append((fic_xml, dest))
                else:
                    # tous les dictionnaires présents
                    for fic_xml in glob(os.path.join(DEST_DIR, name, '*.xml')):
                        copies.append((fic_xml, dest))
            else:
                print u"** %s : destination inexistante (%s)**" % (name, dest)

# dictionnaires communs
for pack_name in CONF_PACKS:
    if pack_name.startswith('conf-'):
        mod_name = pack_name.lstrip('conf-')
        dest = os.path.join(ZEPHIR_DICT_DIR, "%s-%s/" % (mod_name, EOLE_VERS))
        if os.path.isdir(dest):
            # ajout automatique des dictionnaires de eole-common
            if os.path.isdir(os.path.join(DEST_DIR, 'eole-common')):
                for fic_xml in glob(os.path.join(DEST_DIR, 'eole-common', '*.xml')):
                    if (os.path.basename(fic_xml) != '2_logs_amon.xml') or (mod_name in ('amon', 'amonecole')):
                        copies.append((fic_xml, dest))
            # ajout des dictionnaires eole-sso
            if mod_name not in ('zephirlogs', 'eolevz', 'eolebase', 'preludemanager'):
                if os.path.isdir(os.path.join(DEST_DIR, 'eole-sso')):
                    for fic_xml in glob(os.path.join(DEST_DIR, 'eole-sso', '*.xml')):
                        copies.append((fic_xml, dest))

# cas d'amonecole : recopie des dictionnaires amon
src_dir = os.path.join(ZEPHIR_DICT_DIR, 'amon-2.2')
if os.path.isdir(src_dir):
    dest = os.path.join(ZEPHIR_DICT_DIR, "amonecole-2.2")
    for fic_xml in glob(os.path.join(src_dir, '*.xml')):
        copies.append((fic_xml, dest))

# lancement des copies
print u"\nCopie des dictionnaires dans le dépot local de zephir-parc ..."
for src, dst in copies:
    # print u" %s --> %s" % (src, dst)
    run_cmd(["/bin/cp", "-f", src, dst])

shutil.rmtree(DEST_DIR)

print "\nVérification de l'état des fichiers (svn st) :\n"
os.chdir(ZEPHIR_DICT_DIR)
print "***", os.getcwd()
run_cmd(["/usr/bin/git", "status"], True)
